"use strict";
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.IntervalWindowCoder = exports.NullableCoder = exports.BoolCoder = exports.DoubleCoder = exports.VarIntCoder = exports.StrUtf8Coder = void 0;
const _tct_flatten_destructured_imports_1 = __importStar(require("../proto/beam_runner_api"));
var runnerApi = _tct_flatten_destructured_imports_1;
const _tct_flatten_destructured_imports_2 = __importStar(require("protobufjs"));
var Reader = _tct_flatten_destructured_imports_2.Reader;
var Writer = _tct_flatten_destructured_imports_2.Writer;
const _tct_flatten_destructured_imports_3 = __importStar(require("./coders"));
var Coder = _tct_flatten_destructured_imports_3.Coder;
var Context = _tct_flatten_destructured_imports_3.Context;
var ProtoContext = _tct_flatten_destructured_imports_3.ProtoContext;
var globalRegistry = _tct_flatten_destructured_imports_3.globalRegistry;
var writeRawBytes = _tct_flatten_destructured_imports_3.writeRawBytes;
const _tct_flatten_destructured_imports_4 = __importStar(require("./required_coders"));
var BytesCoder = _tct_flatten_destructured_imports_4.BytesCoder;
var InstantCoder = _tct_flatten_destructured_imports_4.InstantCoder;
const long_1 = __importStar(require("long")), _tct_flatten_destructured_imports_5 = long_1;
var Long = _tct_flatten_destructured_imports_5.default;
const _tct_flatten_destructured_imports_6 = __importStar(require("../values"));
var IntervalWindow = _tct_flatten_destructured_imports_6.IntervalWindow;
// Historical
__exportStar(require("./required_coders"), exports);
/**
 * @fileoverview Defines all of the Apache Beam standard coders.
 *
 * Beyond required coders, standard coders provide an efficient way to encode
 * data for communication between the runner and various Beam workers for
 * types that commonly cross process boundaries. Though none of these coders
 * is strictly necessary, if encodings are given for these types it is highly
 * advised to use these definitions that are interoperable with runners and
 * other SDKs.
 *
 * For the schema-aware transform RowCoder, which is a coder for rows of data
 * with a predetermined schema, it is also advised.
 *
 * The formal specifications for these coders can be found in
 * model/pipeline/src/main/proto/beam_runner_api.proto
 */
class StrUtf8Coder {
    constructor() {
        this.type = "stringutf8coder";
        this.encoder = new TextEncoder();
        this.decoder = new TextDecoder();
    }
    encode(element, writer, context) {
        const encodedElement = this.encoder.encode(element);
        BytesCoder.INSTANCE.encode(encodedElement, writer, context);
    }
    decode(reader, context) {
        return this.decoder.decode(BytesCoder.INSTANCE.decode(reader, context));
    }
    toProto() {
        return {
            spec: {
                urn: StrUtf8Coder.URN,
                payload: new Uint8Array(),
            },
            componentCoderIds: [],
        };
    }
}
exports.StrUtf8Coder = StrUtf8Coder;
StrUtf8Coder.URN = "beam:coder:string_utf8:v1";
globalRegistry().register(StrUtf8Coder.URN, StrUtf8Coder);
class VarIntCoder {
    constructor() {
        this.type = "varintcoder";
    }
    encode(element, writer) {
        writer.int32(element);
    }
    decode(reader) {
        return reader.int32();
    }
    toProto() {
        return {
            spec: {
                urn: VarIntCoder.URN,
                payload: new Uint8Array(),
            },
            componentCoderIds: [],
        };
    }
}
exports.VarIntCoder = VarIntCoder;
VarIntCoder.URN = "beam:coder:varint:v1";
VarIntCoder.INSTANCE = new VarIntCoder();
globalRegistry().register(VarIntCoder.URN, VarIntCoder);
class DoubleCoder {
    encode(element, writer) {
        const farr = new Float64Array([element]);
        const barr = new Uint8Array(farr.buffer).reverse();
        writeRawBytes(barr, writer);
    }
    decode(reader) {
        const barr = new Uint8Array(reader.buf);
        const dView = new DataView(barr.buffer.slice(reader.pos, reader.pos + 8));
        reader.double();
        return dView.getFloat64(0, false);
    }
    toProto() {
        return {
            spec: {
                urn: DoubleCoder.URN,
                payload: new Uint8Array(),
            },
            componentCoderIds: [],
        };
    }
}
exports.DoubleCoder = DoubleCoder;
DoubleCoder.URN = "beam:coder:double:v1";
globalRegistry().register(DoubleCoder.URN, DoubleCoder);
class BoolCoder {
    constructor() {
        this.type = "boolcoder";
    }
    encode(element, writer) {
        writer.bool(element);
    }
    decode(reader) {
        return reader.bool();
    }
    toProto() {
        return {
            spec: {
                urn: BoolCoder.URN,
                payload: new Uint8Array(),
            },
            componentCoderIds: [],
        };
    }
}
exports.BoolCoder = BoolCoder;
BoolCoder.URN = "beam:coder:bool:v1";
globalRegistry().register(BoolCoder.URN, BoolCoder);
class NullableCoder {
    constructor(elementCoder) {
        this.type = "nullablecoder";
        this.elementCoder = elementCoder;
    }
    encode(element, writer, context) {
        if (element === null || element === undefined) {
            writer.bool(false);
        }
        else {
            writer.bool(true);
            this.elementCoder.encode(element, writer, context);
        }
    }
    decode(reader, context) {
        if (reader.bool()) {
            return this.elementCoder.decode(reader, context);
        }
        else {
            return undefined;
        }
    }
    toProto(pipelineContext) {
        return {
            spec: {
                urn: NullableCoder.URN,
                payload: new Uint8Array(),
            },
            componentCoderIds: [pipelineContext.getCoderId(this.elementCoder)],
        };
    }
}
exports.NullableCoder = NullableCoder;
NullableCoder.URN = "beam:coder:nullable:v1";
globalRegistry().register(NullableCoder.URN, NullableCoder);
class IntervalWindowCoder {
    encode(value, writer, context) {
        InstantCoder.INSTANCE.encode(value.end, writer, context);
        writer.int64(value.end.sub(value.start));
    }
    decode(reader, context) {
        const end = InstantCoder.INSTANCE.decode(reader, context);
        const duration = reader.int64();
        return new IntervalWindow(end.sub(duration), end);
    }
    toProto() {
        return {
            spec: {
                urn: IntervalWindowCoder.URN,
                payload: new Uint8Array(),
            },
            componentCoderIds: [],
        };
    }
}
exports.IntervalWindowCoder = IntervalWindowCoder;
IntervalWindowCoder.URN = "beam:coder:interval_window:v1";
IntervalWindowCoder.INSTANCE = new IntervalWindowCoder();
globalRegistry().register(IntervalWindowCoder.URN, IntervalWindowCoder);
const _tct_flatten_destructured_imports_7 = __importStar(require("../serialization"));
var requireForSerialization = _tct_flatten_destructured_imports_7.requireForSerialization;
requireForSerialization("apache-beam/coders/standard_coders", exports);
//# sourceMappingURL=standard_coders.js.map