"use strict";
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
/**
 * Downloads the required artifacts from the service to the destination
 * directory.
 */
async function* resolveArtifacts(client, artifacts, localDir = defaultArtifactDir) {
    async function storeArtifact(artifact) {
        if (artifact.typeUrn === "beam:artifact:type:file:v1") {
            const payload = runnerApi.ArtifactFilePayload.fromBinary(artifact.typePayload);
            // As we're storing artifacts by hash, we can safely re-use if we've
            // ever seen this before.
            if (fs.existsSync(payload.sha256 && path.join(localDir, payload.sha256))) {
                return {
                    typeUrn: "beam:artifact:type:file:v1",
                    typePayload: runnerApi.ArtifactFilePayload.toBinary({
                        path: path.join(localDir, payload.sha256),
                        sha256: payload.sha256,
                    }),
                    roleUrn: artifact.roleUrn,
                    rolePayload: artifact.rolePayload,
                };
            }
            // TODO: (Perf) Hardlink if same filesystem and correct hash?
        }
        fs.mkdirSync(localDir, { recursive: true });
        const tmpName = tempFile(localDir);
        const tmpHandle = fs.createWriteStream(tmpName);
        const call = client.getArtifact({ artifact: artifact });
        const hasher = crypto.createHash("sha256");
        call.responses.onMessage((_tct_transform_1 = (msg) => {
            hasher.update(msg.data);
            tmpHandle.write(msg.data);
        }, _tct_transform_1.__closure = () => ({ hasher, tmpHandle }), _tct_transform_1));
        await call;
        tmpHandle.close();
        const hash = hasher.digest("hex");
        const finalPath = path.join(localDir, hash);
        fs.renameSync(tmpName, finalPath);
        return {
            typeUrn: "beam:artifact:type:file:v1",
            typePayload: runnerApi.ArtifactFilePayload.toBinary({
                path: finalPath,
                sha256: hash,
            }),
            roleUrn: artifact.roleUrn,
            rolePayload: artifact.rolePayload,
        };
    }
    storeArtifact.__closure = () => ({ runnerApi, fs, path, localDir, tempFile, client, crypto });
    const resolved = await client.resolveArtifacts({
        artifacts: Array.from(artifacts),
        preferredUrns: [],
    }).response;
    for (const artifact of resolved.replacements) {
        if (artifact.typeUrn === "beam:artifact:type:url:v1" ||
            artifact.typeUrn === "beam:artifact:type:embedded:v1") {
            // TODO: (Typescript) Yield from asycn?
            yield artifact;
        }
        else {
            yield await storeArtifact(artifact);
        }
    }
}
var _tct_transform_1, _tct_transform_2;
resolveArtifacts.__closure = () => ({ runnerApi, fs, path, tempFile, crypto, Array });
async function offerArtifacts(client, stagingToken, rootDir = defaultArtifactDir) {
    const call = client.reverseArtifactRetrievalService();
    call.responses.onMessage((_tct_transform_2 = async (msg) => {
        switch (msg.request.oneofKind) {
            case "resolveArtifact":
                call.requests.send({
                    stagingToken: stagingToken,
                    isLast: false,
                    response: {
                        oneofKind: "resolveArtifactResponse",
                        resolveArtifactResponse: {
                            replacements: msg.request.resolveArtifact.artifacts,
                        },
                    },
                });
                break;
            case "getArtifact":
                switch (msg.request.getArtifact.artifact.typeUrn) {
                    case "beam:artifact:type:file:v1":
                        const payload = runnerApi.ArtifactFilePayload.fromBinary(msg.request.getArtifact.artifact.typePayload);
                        const filePath = path.normalize(payload.path);
                        if (!filePath.startsWith(rootDir)) {
                            throw new Error("Refusing to serve " +
                                filePath +
                                " as it is not under " +
                                rootDir);
                        }
                        const handle = fs.createReadStream(filePath);
                        for await (const chunk of handle) {
                            call.requests.send({
                                stagingToken: stagingToken,
                                isLast: false,
                                response: {
                                    oneofKind: "getArtifactResponse",
                                    getArtifactResponse: { data: chunk },
                                },
                            });
                        }
                        call.requests.send({
                            stagingToken: stagingToken,
                            isLast: true,
                            response: {
                                oneofKind: "getArtifactResponse",
                                getArtifactResponse: { data: new Uint8Array() },
                            },
                        });
                        break;
                    default:
                        throw new Error("Unknown artifact type " +
                            msg.request.getArtifact.artifact.typeUrn);
                }
                break;
            default:
                throw new Error("Unknown request type " + msg.request.oneofKind);
        }
    }, _tct_transform_2.__closure = () => ({ call, stagingToken, runnerApi, path, rootDir, Error, fs, Uint8Array }), _tct_transform_2));
    call.requests.send({
        stagingToken: stagingToken,
        isLast: false,
        response: {
            oneofKind: undefined,
        },
    });
    await call;
}
offerArtifacts.__closure = () => ({ runnerApi, path, Error, fs, Uint8Array });
function tempFile(dir) {
    return path.join(dir, "temp" +
        crypto
            .randomBytes(16)
            .toString("base64")
            .replace(/[^a-zA-Z0-9]/g, "_"));
}
tempFile.__closure = () => ({ path, crypto });
Object.defineProperty(exports, "__esModule", { value: true });
exports.offerArtifacts = exports.resolveArtifacts = void 0;
var crypto = require("crypto");
var fs = require("fs");
var path = require("path");
var os = require("os");
const _tct_flatten_destructured_imports_1 = __importStar(require("../proto/beam_runner_api"));
var runnerApi = _tct_flatten_destructured_imports_1;
const _tct_flatten_destructured_imports_2 = __importStar(require("../proto/beam_artifact_api"));
var artifactApi = _tct_flatten_destructured_imports_2;
const _tct_flatten_destructured_imports_3 = __importStar(require("../proto/beam_artifact_api.client"));
var IArtifactRetrievalServiceClient = _tct_flatten_destructured_imports_3.IArtifactRetrievalServiceClient;
var IArtifactStagingServiceClient = _tct_flatten_destructured_imports_3.IArtifactStagingServiceClient;
const defaultArtifactDir = path.join(os.homedir(), ".apache_beam", "cache", "artifacts");
exports.resolveArtifacts = resolveArtifacts;
exports.offerArtifacts = offerArtifacts;
//# sourceMappingURL=artifacts.js.map