"use strict";
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
/**
 * Runs an SQL statement over a set of input PCollection(s).
 *
 * The input can either be a single PCollection, in which case the table is
 * named PCOLLECTION, or an object with PCollection values, in which case the
 * corresponding names can be used in the sql statement.
 *
 * The input(s) must be schema'd (i.e. use the RowCoder). This can be done
 * by explicitly setting the schema with external.withRowCoder or passing
 * a prototype element in as a second argument, e.g.
 *
 * pcoll.applyAsync(
 *    sqlTransform(
 *        "select a, b from PCOLLECTION",
 *        {a: 0, b: "string"},
 *    ));
 */
function sqlTransform(query, inputTypes = null) {
    // TOOD: (API) (Typescript): How to infer input_types, or at least make it optional.
    async function expandInternal(input) {
        function withCoder(pcoll, type) {
            if (type) {
                if (!(pcoll.pipeline.context.getPCollectionCoder(pcoll) instanceof
                    row_coder.RowCoder)) {
                    throw new Error("SqlTransform can only be applied to schema'd transforms. " +
                        "Please ensure the input PCollection(s) have a RowCoder, " +
                        "or pass a prototypical element in as the second argument " +
                        "of SqlTransform so that one can be inferred.");
                }
                return pcoll;
            }
            return pcoll.apply(internal.withRowCoder(type));
        }
        withCoder.__closure = () => ({ row_coder, Error, internal });
        if (input instanceof PCollection) {
            input = withCoder(input, inputTypes);
        }
        else {
            input = Object.fromEntries(Object.keys(input).map((_tct_transform_1 = (tag) => [
                tag,
                withCoder(input[tag], inputTypes === null || inputTypes === undefined
                    ? null
                    : inputTypes[tag]),
            ], _tct_transform_1.__closure = () => ({ withCoder, input, inputTypes }), _tct_transform_1)));
        }
        return await P(input).applyAsync(external.rawExternalTransform("beam:external:java:sql:v1", { query: query }, serviceProviderFromJavaGradleTarget("sdks:java:extensions:sql:expansion-service:shadowJar")));
    }
    expandInternal.__closure = () => ({ row_coder, Error, internal, PCollection, inputTypes, Object, P, external, query, serviceProviderFromJavaGradleTarget });
    return transform.withName(`sqlTransform(${query})`, expandInternal);
}
var _tct_transform_1;
sqlTransform.__closure = () => ({ row_coder, Error, internal, PCollection, Object, P, external, serviceProviderFromJavaGradleTarget, transform });
Object.defineProperty(exports, "__esModule", { value: true });
exports.sqlTransform = void 0;
const _tct_flatten_destructured_imports_1 = __importStar(require("./external"));
var external = _tct_flatten_destructured_imports_1;
const _tct_flatten_destructured_imports_2 = __importStar(require("./internal"));
var internal = _tct_flatten_destructured_imports_2;
const _tct_flatten_destructured_imports_3 = __importStar(require("./transform"));
var transform = _tct_flatten_destructured_imports_3;
const _tct_flatten_destructured_imports_4 = __importStar(require("../coders/row_coder"));
var row_coder = _tct_flatten_destructured_imports_4;
const _tct_flatten_destructured_imports_5 = __importStar(require("../pvalue"));
var P = _tct_flatten_destructured_imports_5.P;
var PCollection = _tct_flatten_destructured_imports_5.PCollection;
const _tct_flatten_destructured_imports_6 = __importStar(require("../utils/service"));
var serviceProviderFromJavaGradleTarget = _tct_flatten_destructured_imports_6.serviceProviderFromJavaGradleTarget;
exports.sqlTransform = sqlTransform;
//# sourceMappingURL=sql.js.map