"use strict";
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
function createLoggingChannel(workerId, endpoint) {
    function toEntry(line, severity) {
        const now_ms = Date.now();
        const seconds = Math.trunc(now_ms / 1000);
        const stageInfo = exports.loggingLocalStorage.getStore();
        return LogEntry.create({
            severity,
            message: line,
            timestamp: {
                seconds: BigInt(seconds),
                nanos: Math.trunc((now_ms - seconds * 1000) * 1e6),
            },
            instructionId: stageInfo?.instructionId,
            transformId: stageInfo?.transformId,
        });
    }
    toEntry.__closure = () => ({ Date, Math, loggingLocalStorage: exports.loggingLocalStorage, LogEntry, BigInt });
    // The createLogMethod is explicitly called to capture the
    // iteration-specific closure.
    function createLogMethod(method, level) {
        const original = console[method];
        return _tct_transform_1 = function () {
            const old = currentConsoleLogLevel.value;
            try {
                currentConsoleLogLevel.value = level;
                original(...arguments);
            }
            finally {
                currentConsoleLogLevel.value = old;
            }
        }, _tct_transform_1.__closure = () => ({ currentConsoleLogLevel, level, original }), _tct_transform_1;
    }
    createLogMethod.__closure = () => ({ console, currentConsoleLogLevel });
    function guessLogLevel(line) {
        if (currentConsoleLogLevel.value !== undefined) {
            return currentConsoleLogLevel.value;
        }
        else {
            try {
                const structuredLog = JSON.parse(line);
                if (structuredLog.level !== undefined) {
                    if (0 <= structuredLog.level && structuredLog.level <= 7) {
                        // Assume https://www.rfc-editor.org/rfc/rfc5424
                        return [
                            LogEntry_Severity_Enum.CRITICAL,
                            LogEntry_Severity_Enum.CRITICAL,
                            LogEntry_Severity_Enum.CRITICAL,
                            LogEntry_Severity_Enum.ERROR,
                            LogEntry_Severity_Enum.WARN,
                            LogEntry_Severity_Enum.INFO,
                            LogEntry_Severity_Enum.INFO,
                            LogEntry_Severity_Enum.DEBUG,
                        ][structuredLog.level];
                    }
                    else {
                        // Something like pinojs.
                        const truncatedLogLevel = Math.floor(structuredLog.level / 10);
                        if (1 <= truncatedLogLevel && truncatedLogLevel <= 7) {
                            return [
                                LogEntry_Severity_Enum.TRACE,
                                LogEntry_Severity_Enum.TRACE,
                                LogEntry_Severity_Enum.DEBUG,
                                LogEntry_Severity_Enum.INFO,
                                LogEntry_Severity_Enum.WARN,
                                LogEntry_Severity_Enum.ERROR,
                                LogEntry_Severity_Enum.CRITICAL,
                                LogEntry_Severity_Enum.CRITICAL,
                            ][truncatedLogLevel];
                        }
                    }
                }
            }
            catch (e) { }
            return LogEntry_Severity_Enum.INFO;
        }
    }
    guessLogLevel.__closure = () => ({ currentConsoleLogLevel, JSON, LogEntry_Severity_Enum, Math });
    const logQueue = new Queue();
    let currentConsoleLogLevel = { value: undefined };
    currentConsoleLogLevel.value = undefined;
    const consoleLogLevels = {
        // Don't include log as other loggers may use it; let it default to INFO.
        error: LogEntry_Severity_Enum.ERROR,
        warn: LogEntry_Severity_Enum.WARN,
        info: LogEntry_Severity_Enum.INFO,
        debug: LogEntry_Severity_Enum.DEBUG,
    };
    for (const [method, level] of Object.entries(consoleLogLevels)) {
        console[method] = createLogMethod(method, level);
    }
    startCapture(process.stdout, (_tct_transform_2 = (out) => logQueue.enqueue(toEntry(out, guessLogLevel(out))), _tct_transform_2.__closure = () => ({ logQueue, toEntry, guessLogLevel }), _tct_transform_2));
    startCapture(process.stderr, (_tct_transform_3 = (out) => logQueue.enqueue(toEntry(out, guessLogLevel(out))), _tct_transform_3.__closure = () => ({ logQueue, toEntry, guessLogLevel }), _tct_transform_3));
    const metadata = new grpc.Metadata();
    metadata.add("worker_id", workerId);
    const client = new BeamFnLoggingClient(endpoint, grpc.ChannelCredentials.createInsecure(), {}, {});
    const channel = client.logging(metadata);
    return _tct_transform_4 = async function () {
        var _tct_transform_5;
        while (true) {
            if (logQueue.length) {
                const messages = [];
                while (logQueue.length && messages.length < 100) {
                    messages.push(logQueue.dequeue());
                }
                await channel.write({ logEntries: messages });
            }
            else {
                await new Promise((_tct_transform_5 = (r) => setTimeout(r, 100), _tct_transform_5.__closure = () => ({ setTimeout }), _tct_transform_5));
            }
        }
    }, _tct_transform_4.__closure = () => ({ logQueue, channel, Promise, setTimeout }), _tct_transform_4;
}
var _tct_transform_1, _tct_transform_2, _tct_transform_3, _tct_transform_4;
createLoggingChannel.__closure = () => ({ Date, Math, loggingLocalStorage: exports.loggingLocalStorage, LogEntry, BigInt, console, currentConsoleLogLevel, JSON, LogEntry_Severity_Enum, Queue, Object, startCapture, process, grpc, BeamFnLoggingClient, Promise, setTimeout });
Object.defineProperty(exports, "__esModule", { value: true });
exports.createLoggingChannel = exports.LoggingStageInfoHolder = exports.loggingLocalStorage = void 0;
const _tct_flatten_destructured_imports_1 = __importStar(require("node:async_hooks"));
var AsyncLocalStorage = _tct_flatten_destructured_imports_1.AsyncLocalStorage;
var AsyncResource = _tct_flatten_destructured_imports_1.AsyncResource;
exports.loggingLocalStorage = new AsyncLocalStorage();
const _tct_flatten_destructured_imports_2 = __importStar(require("@grpc/grpc-js"));
var grpc = _tct_flatten_destructured_imports_2;
const _tct_flatten_destructured_imports_3 = __importStar(require("capture-console"));
var startCapture = _tct_flatten_destructured_imports_3.startCapture;
const _tct_flatten_destructured_imports_4 = __importStar(require("queue-typescript"));
var Queue = _tct_flatten_destructured_imports_4.Queue;
const _tct_flatten_destructured_imports_5 = __importStar(require("../proto/beam_fn_api"));
var LogEntry = _tct_flatten_destructured_imports_5.LogEntry;
var LogEntry_Severity_Enum = _tct_flatten_destructured_imports_5.LogEntry_Severity_Enum;
const _tct_flatten_destructured_imports_6 = __importStar(require("../proto/beam_fn_api.grpc-client"));
var BeamFnLoggingClient = _tct_flatten_destructured_imports_6.BeamFnLoggingClient;
class LoggingStageInfoHolder {
}
exports.LoggingStageInfoHolder = LoggingStageInfoHolder;
exports.createLoggingChannel = createLoggingChannel;
//# sourceMappingURL=logging.js.map