"use strict";
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
function Uint8ArrayConcat(chunks) {
    if (chunks.length === 1) {
        // (Very) common case.
        return chunks[0];
    }
    else if (chunks.length === 0) {
        return new Uint8Array();
    }
    else {
        const fullData = new Uint8Array(chunks.map((chunk) => chunk.length).reduce((a, b) => a + b));
        let start = 0;
        for (const chunk of chunks) {
            fullData.set(chunk, start);
            start += chunk.length;
        }
        return fullData;
    }
}
Uint8ArrayConcat.__closure = () => ({ Uint8Array });
Object.defineProperty(exports, "__esModule", { value: true });
exports.Uint8ArrayConcat = exports.MultiplexingStateChannel = exports.GrpcStateProvider = exports.CachingStateProvider = void 0;
const _tct_flatten_destructured_imports_1 = __importStar(require("@grpc/grpc-js"));
var grpc = _tct_flatten_destructured_imports_1;
const _tct_flatten_destructured_imports_2 = __importStar(require("../proto/beam_fn_api"));
var fnApi = _tct_flatten_destructured_imports_2;
const _tct_flatten_destructured_imports_3 = __importStar(require("../proto/beam_fn_api.grpc-client"));
var BeamFnStateClient = _tct_flatten_destructured_imports_3.BeamFnStateClient;
// TODO: (Advanced) Cross-bundle caching.
class CachingStateProvider {
    constructor(underlying) {
        this.cache = new Map();
        this.underlying = underlying;
    }
    getState(stateKey, decode) {
        var _tct_transform_1;
        // TODO: (Perf) Consider caching on something ligher-weight than the full
        // serialized key, only constructing this proto when interacting with
        // the runner.
        const cacheKey = Buffer.from(fnApi.StateKey.toBinary(stateKey)).toString("base64");
        if (this.cache.has(cacheKey)) {
            return this.cache.get(cacheKey);
        }
        let result = this.underlying.getState(stateKey, decode);
        const this_ = this;
        if (result.type === "promise") {
            result = {
                type: "promise",
                promise: result.promise.then((_tct_transform_1 = (value) => {
                    this_.cache.set(cacheKey, { type: "value", value });
                    return value;
                }, _tct_transform_1.__closure = () => ({ this_, cacheKey }), _tct_transform_1)),
            };
        }
        // TODO: (Perf) Cache eviction.
        this.cache.set(cacheKey, result);
        return result;
    }
}
exports.CachingStateProvider = CachingStateProvider;
class GrpcStateProvider {
    constructor(multiplexingChannel, instructionId) {
        this.multiplexingChannel = multiplexingChannel;
        this.instructionId = instructionId;
    }
    getState(stateKey, decode) {
        return this.multiplexingChannel.getState(this.instructionId, stateKey, decode);
    }
}
exports.GrpcStateProvider = GrpcStateProvider;
/**
 * This class manages a single state channel that can be shared across multiple
 * bundles.
 *
 * Per the Beam state protocol [link], every state request is tagged with a
 * unique id which is then part of the (possibly out of order) response.
 * We model that by registering callbacks associated with each id that
 * resolve the corresponding promise that was created when the request was sent.
 */
class MultiplexingStateChannel {
    constructor(endpoint, workerId) {
        var _tct_transform_2;
        this.callbacks = new Map();
        this.idCounter = 0;
        this.stateClient = new BeamFnStateClient(endpoint, grpc.ChannelCredentials.createInsecure(), {}, {});
        const metadata = new grpc.Metadata();
        metadata.add("worker_id", workerId);
        this.stateChannel = this.stateClient.state(metadata);
        const this_ = this;
        this.stateChannel.on("data", (_tct_transform_2 = (response) => {
            const cb = this_.callbacks.get(response.id);
            this_.callbacks.delete(response.id);
            cb(response);
        }, _tct_transform_2.__closure = () => ({ this_ }), _tct_transform_2));
        this.stateChannel.on("error", (error) => {
            this.error = error;
        });
    }
    close() {
        this.closed = true;
        this.stateChannel.end();
    }
    getState(instructionId, stateKey, decode) {
        // Not inlined as it may need to be called recursively to handle
        // continuation tokens.
        function responseCallback(resolve, reject, prevChunks = []) {
            return _tct_transform_3 = (response) => {
                if (this_.error) {
                    reject(this_.error);
                    return;
                }
                if (response.error) {
                    reject(response.error);
                    return;
                }
                let getResponse;
                if (response.response.oneofKind === "get") {
                    getResponse = response.response.get;
                }
                else {
                    reject("Expected get response " + response.response.oneofKind);
                    return;
                }
                // We have a properly formed response.
                const allChunks = prevChunks.concat([getResponse.data]);
                if (getResponse.continuationToken &&
                    getResponse.continuationToken.length > 0) {
                    // Register another callback to fetch the rest of the data.
                    const continueId = "continueStateRequest" + this_.idCounter++;
                    this_.callbacks.set(continueId, responseCallback(resolve, reject, allChunks));
                    this_.stateChannel.write({
                        id: continueId,
                        instructionId,
                        stateKey,
                        request: {
                            oneofKind: "get",
                            get: {
                                continuationToken: getResponse.continuationToken,
                            },
                        },
                    });
                }
                else {
                    // End of the data stream; resolve with everything we got.
                    resolve(decode(Uint8ArrayConcat(allChunks)));
                }
            }, _tct_transform_3.__closure = () => ({ this_, reject, prevChunks, responseCallback, resolve, instructionId, stateKey, decode, Uint8ArrayConcat }), _tct_transform_3;
        }
        var _tct_transform_3, _tct_transform_4;
        responseCallback.__closure = () => ({ this_, instructionId, stateKey, decode, Uint8ArrayConcat });
        if (this.closed) {
            throw new Error("State stream is closed.");
        }
        else if (this.error) {
            throw this.error;
        }
        const this_ = this;
        const id = "stateRequest" + this.idCounter++;
        const promise = new Promise((_tct_transform_4 = (resolve, reject) => {
            this_.callbacks.set(id, responseCallback(resolve, reject));
        }, _tct_transform_4.__closure = () => ({ this_, id, responseCallback }), _tct_transform_4));
        this.stateChannel.write({
            id,
            instructionId,
            stateKey,
            request: {
                oneofKind: "get",
                get: fnApi.StateGetRequest.create({}),
            },
        });
        return { type: "promise", promise };
    }
}
exports.MultiplexingStateChannel = MultiplexingStateChannel;
exports.Uint8ArrayConcat = Uint8ArrayConcat;
//# sourceMappingURL=state.js.map