"use strict";
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var _tct_transform_1;
Object.defineProperty(exports, "__esModule", { value: true });
const _tct_flatten_destructured_imports_1 = __importStar(require("../src/apache_beam"));
var beam = _tct_flatten_destructured_imports_1;
const _tct_flatten_destructured_imports_2 = __importStar(require("../src/apache_beam/runners/runner"));
var createRunner = _tct_flatten_destructured_imports_2.createRunner;
const _tct_flatten_destructured_imports_3 = __importStar(require("../src/apache_beam/testing/assert"));
var testing = _tct_flatten_destructured_imports_3;
const _tct_flatten_destructured_imports_4 = __importStar(require("../src/apache_beam/values"));
var KV = _tct_flatten_destructured_imports_4.KV;
const _tct_flatten_destructured_imports_5 = __importStar(require("../src/apache_beam/transforms/combiners"));
var combiners = _tct_flatten_destructured_imports_5;
const _tct_flatten_destructured_imports_6 = __importStar(require("../src/apache_beam/transforms/group_and_combine"));
var CombineFn = _tct_flatten_destructured_imports_6.CombineFn;
var groupBy = _tct_flatten_destructured_imports_6.groupBy;
var groupGlobally = _tct_flatten_destructured_imports_6.groupGlobally;
var countPerElement = _tct_flatten_destructured_imports_6.countPerElement;
var countGlobally = _tct_flatten_destructured_imports_6.countGlobally;
describe("Apache Beam combiners", (_tct_transform_1 = function () {
    var _tct_transform_2, _tct_transform_3, _tct_transform_4, _tct_transform_5, _tct_transform_6, _tct_transform_7, _tct_transform_8;
    it("runs wordcount with a countPerKey transform and asserts the result", (_tct_transform_2 = async function () {
        var _tct_transform_9;
        await createRunner().run((_tct_transform_9 = (root) => {
            const lines = root.apply(beam.create([
                "In the beginning God created the heaven and the earth.",
                "And the earth was without form, and void; and darkness was upon the face of the deep.",
                "And the Spirit of God moved upon the face of the waters.",
                "And God said, Let there be light: and there was light.",
            ]));
            lines
                .map((s) => s.toLowerCase())
                .flatMap(function* splitWords(line) {
                yield* line.split(/[^a-z]+/);
            })
                .map((elm) => ({ key: elm, value: 1 }))
                .apply(groupBy("key").combining("value", combiners.sum, "value"))
                .apply(testing.assertDeepEqual([
                { key: "in", value: 1 },
                { key: "the", value: 9 },
                { key: "beginning", value: 1 },
                { key: "god", value: 3 },
                { key: "created", value: 1 },
                { key: "heaven", value: 1 },
                { key: "and", value: 7 },
                { key: "earth", value: 2 },
                { key: "", value: 4 },
                { key: "was", value: 3 },
                { key: "without", value: 1 },
                { key: "form", value: 1 },
                { key: "void", value: 1 },
                { key: "darkness", value: 1 },
                { key: "upon", value: 2 },
                { key: "face", value: 2 },
                { key: "of", value: 3 },
                { key: "deep", value: 1 },
                { key: "spirit", value: 1 },
                { key: "moved", value: 1 },
                { key: "waters", value: 1 },
                { key: "said", value: 1 },
                { key: "let", value: 1 },
                { key: "there", value: 2 },
                { key: "be", value: 1 },
                { key: "light", value: 2 },
            ]));
        }, _tct_transform_9.__closure = () => ({ beam, groupBy, combiners, testing }), _tct_transform_9));
    }, _tct_transform_2.__closure = () => ({ createRunner, beam, groupBy, combiners, testing }), _tct_transform_2));
    it("runs wordcount with a countGlobally transform and asserts the result", (_tct_transform_3 = async function () {
        var _tct_transform_10;
        await createRunner().run((_tct_transform_10 = (root) => {
            const lines = root.apply(beam.create(["And God said, Let there be light: and there was light"]));
            lines
                .map((s) => s.toLowerCase())
                .flatMap(function* splitWords(line) {
                yield* line.split(/[^a-z]+/);
            })
                .apply(countGlobally())
                .apply(testing.assertDeepEqual([11]));
        }, _tct_transform_10.__closure = () => ({ beam, countGlobally, testing }), _tct_transform_10));
    }, _tct_transform_3.__closure = () => ({ createRunner, beam, countGlobally, testing }), _tct_transform_3));
    it("runs an example where a custom combine function is passed", (_tct_transform_4 = async function () {
        function unstableStdDevCombineFn() {
            // NOTE: This Standard Deviation algorithm is **unstable**, so it is not recommended
            //    for an actual production-level pipeline.
            return {
                createAccumulator: function () {
                    return { count: 0, sum: 0, sumOfSquares: 0 };
                },
                addInput: function (acc, inp) {
                    return {
                        count: acc.count + 1,
                        sum: acc.sum + inp,
                        sumOfSquares: acc.sumOfSquares + inp * inp,
                    };
                },
                mergeAccumulators: function (accumulators) {
                    return accumulators.reduce((previous, current) => ({
                        count: previous.count + current.count,
                        sum: previous.sum + current.sum,
                        sumOfSquares: previous.sumOfSquares + current.sumOfSquares,
                    }));
                },
                extractOutput: function (acc) {
                    const mean = acc.sum / acc.count;
                    return acc.sumOfSquares / acc.count - mean * mean;
                },
            };
        }
        var _tct_transform_11;
        await createRunner().run((_tct_transform_11 = (root) => {
            const lines = root.apply(beam.create([
                "In the beginning God created the heaven and the earth.",
                "And the earth was without form, and void; and darkness was upon the face of the deep.",
                "And the Spirit of God moved upon the face of the waters.",
                "And God said, Let there be light: and there was light.",
            ]));
            lines
                .map((s) => s.toLowerCase())
                .flatMap(function* splitWords(line) {
                yield* line.split(/[^a-z]+/);
            })
                .map((word) => word.length)
                .apply(groupGlobally()
                .combining((c) => c, combiners.mean, "mean")
                .combining((c) => c, unstableStdDevCombineFn(), "stdDev"))
                .apply(testing.assertDeepEqual([
                { mean: 3.611111111111111, stdDev: 3.2746913580246897 },
            ]));
        }, _tct_transform_11.__closure = () => ({ beam, groupGlobally, combiners, unstableStdDevCombineFn, testing }), _tct_transform_11));
    }, _tct_transform_4.__closure = () => ({ createRunner, beam, groupGlobally, combiners, testing }), _tct_transform_4));
    it("test GroupBy with combining", (_tct_transform_5 = async function () {
        var _tct_transform_12;
        await createRunner().run((_tct_transform_12 = (root) => {
            const inputs = root.apply(beam.create([
                { k: "k1", a: 1, b: 100 },
                { k: "k1", a: 2, b: 200 },
                { k: "k2", a: 9, b: 1000 },
            ]));
            inputs
                .apply(groupBy("k")
                .combining("a", combiners.max, "aMax")
                .combining("a", combiners.sum, "aSum")
                .combining("b", combiners.mean, "mean"))
                .apply(testing.assertDeepEqual([
                { k: "k1", aMax: 2, aSum: 3, mean: 150 },
                { k: "k2", aMax: 9, aSum: 9, mean: 1000 },
            ]));
        }, _tct_transform_12.__closure = () => ({ beam, groupBy, combiners, testing }), _tct_transform_12));
    }, _tct_transform_5.__closure = () => ({ createRunner, beam, groupBy, combiners, testing }), _tct_transform_5));
    it("test GroupBy list with combining", (_tct_transform_6 = async function () {
        var _tct_transform_13;
        await createRunner().run((_tct_transform_13 = (root) => {
            const inputs = root.apply(beam.create([
                { a: 1, b: 10, c: 100 },
                { a: 2, b: 10, c: 100 },
                { a: 1, b: 10, c: 400 },
            ]));
            inputs
                .apply(groupBy(["a", "b"]).combining("c", combiners.sum, "sum"))
                .apply(testing.assertDeepEqual([
                { a: 1, b: 10, sum: 500 },
                { a: 2, b: 10, sum: 100 },
            ]));
            inputs
                .apply(groupBy(["b", "c"]).combining("a", combiners.sum, "sum"))
                .apply(testing.assertDeepEqual([
                { b: 10, c: 100, sum: 3 },
                { b: 10, c: 400, sum: 1 },
            ]));
        }, _tct_transform_13.__closure = () => ({ beam, groupBy, combiners, testing }), _tct_transform_13));
    }, _tct_transform_6.__closure = () => ({ createRunner, beam, groupBy, combiners, testing }), _tct_transform_6));
    it("test GroupBy expr with combining", (_tct_transform_7 = async function () {
        var _tct_transform_14;
        await createRunner().run((_tct_transform_14 = (root) => {
            const inputs = root.apply(beam.create([
                { a: 1, b: 10 },
                { a: 0, b: 20 },
                { a: -1, b: 30 },
            ]));
            inputs
                .apply(groupBy((element) => element.a * element.a).combining("b", combiners.sum, "sum"))
                .apply(testing.assertDeepEqual([
                { key: 1, sum: 40 },
                { key: 0, sum: 20 },
            ]));
        }, _tct_transform_14.__closure = () => ({ beam, groupBy, combiners, testing }), _tct_transform_14));
    }, _tct_transform_7.__closure = () => ({ createRunner, beam, groupBy, combiners, testing }), _tct_transform_7));
    it("test GroupBy with binary combinefn", (_tct_transform_8 = async function () {
        var _tct_transform_15;
        await createRunner().run((_tct_transform_15 = (root) => {
            var _tct_transform_16;
            const inputs = root.apply(beam.create([
                { key: 0, value: 10 },
                { key: 1, value: 20 },
                { key: 0, value: 30 },
            ]));
            inputs
                .apply(groupBy("key")
                .combining("value", (x, y) => x + y, "sum")
                .combining("value", (_tct_transform_16 = (x, y) => Math.max(x, y), _tct_transform_16.__closure = () => ({ Math }), _tct_transform_16), "max"))
                .apply(testing.assertDeepEqual([
                { key: 0, sum: 40, max: 30 },
                { key: 1, sum: 20, max: 20 },
            ]));
        }, _tct_transform_15.__closure = () => ({ beam, groupBy, Math, testing }), _tct_transform_15));
    }, _tct_transform_8.__closure = () => ({ createRunner, beam, groupBy, Math, testing }), _tct_transform_8));
}, _tct_transform_1.__closure = () => ({ it, createRunner, beam, groupBy, combiners, testing, countGlobally, groupGlobally, Math }), _tct_transform_1));
//# sourceMappingURL=combine_test.js.map