"use strict";
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
////////// Helper Functions. //////////
function makePTransform(urn, inputs, outputs, payload = null) {
    return {
        spec: {
            urn: urn,
            payload: new TextEncoder().encode(JSON.stringify(payload)),
        },
        inputs: inputs,
        outputs: outputs,
        uniqueName: "",
        environmentId: "",
        displayData: [],
        annotations: {},
        subtransforms: [],
    };
}
var _tct_transform_1;
makePTransform.__closure = () => ({ TextEncoder, JSON });
Object.defineProperty(exports, "__esModule", { value: true });
const long_1 = __importStar(require("long")), _tct_flatten_destructured_imports_1 = long_1;
var Long = _tct_flatten_destructured_imports_1.default;
const _tct_flatten_destructured_imports_2 = __importStar(require("../src/apache_beam/coders/standard_coders"));
var PaneInfoCoder = _tct_flatten_destructured_imports_2.PaneInfoCoder;
const _tct_flatten_destructured_imports_3 = __importStar(require("../src/apache_beam/proto/beam_runner_api"));
var PTransform = _tct_flatten_destructured_imports_3.PTransform;
var PCollection = _tct_flatten_destructured_imports_3.PCollection;
const _tct_flatten_destructured_imports_4 = __importStar(require("../src/apache_beam/proto/beam_fn_api"));
var ProcessBundleDescriptor = _tct_flatten_destructured_imports_4.ProcessBundleDescriptor;
const _tct_flatten_destructured_imports_5 = __importStar(require("../src/apache_beam/worker/worker"));
var worker = _tct_flatten_destructured_imports_5;
const _tct_flatten_destructured_imports_6 = __importStar(require("../src/apache_beam/worker/operators"));
var operators = _tct_flatten_destructured_imports_6;
const _tct_flatten_destructured_imports_7 = __importStar(require("../src/apache_beam/values"));
var Window = _tct_flatten_destructured_imports_7.Window;
var GlobalWindow = _tct_flatten_destructured_imports_7.GlobalWindow;
var Instant = _tct_flatten_destructured_imports_7.Instant;
var PaneInfo = _tct_flatten_destructured_imports_7.PaneInfo;
var WindowedValue = _tct_flatten_destructured_imports_7.WindowedValue;
const assert = require("assert");
////////// Define some mock operators for use in our tests. //////////
const CREATE_URN = "create";
const RECORDING_URN = "recording";
const PARTITION_URN = "partition";
class Create {
    constructor(transformId, transform, context) {
        this.transformId = transformId;
        this.data = JSON.parse(new TextDecoder().decode(transform.spec.payload));
        this.receivers = Object.values(transform.outputs).map(context.getReceiver);
    }
    process(wvalue) {
        return operators.NonPromise;
    }
    async startBundle() {
        var _tct_transform_2;
        const this_ = this;
        this_.data.forEach((_tct_transform_2 = function (datum) {
            var _tct_transform_3;
            this_.receivers.map((_tct_transform_3 = (receiver) => receiver.receive({
                value: datum,
                windows: [new GlobalWindow()],
                pane: PaneInfoCoder.ONE_AND_ONLY_FIRING,
                timestamp: Long.fromValue("-9223372036854775"),
            }), _tct_transform_3.__closure = () => ({ datum, GlobalWindow, PaneInfoCoder, Long }), _tct_transform_3));
        }, _tct_transform_2.__closure = () => ({ this_, GlobalWindow, PaneInfoCoder, Long }), _tct_transform_2));
    }
    async finishBundle() { }
}
operators.registerOperator(CREATE_URN, Create);
class Recording {
    constructor(transformId, transform, context) {
        this.transformId = transformId;
        this.receivers = Object.values(transform.outputs).map(context.getReceiver);
    }
    static reset() {
        Recording.log = [];
    }
    async startBundle() {
        Recording.log.push(this.transformId + ".startBundle()");
    }
    process(wvalue) {
        var _tct_transform_4, _tct_transform_5;
        Recording.log.push(this.transformId + ".process(" + wvalue.value + ")");
        const results = this.receivers.map((_tct_transform_4 = (receiver) => receiver.receive(wvalue), _tct_transform_4.__closure = () => ({ wvalue }), _tct_transform_4));
        if (!results.every((_tct_transform_5 = (r) => r === operators.NonPromise, _tct_transform_5.__closure = () => ({ operators }), _tct_transform_5))) {
            throw new Error("Unexpected non-promise: " + results);
        }
        return operators.NonPromise;
    }
    async finishBundle() {
        Recording.log.push(this.transformId + ".finishBundle()");
    }
}
operators.registerOperator(RECORDING_URN, Recording);
worker.primitiveSinks.push(RECORDING_URN);
class Partition {
    constructor(transformId, transform, context) {
        this.transformId = transformId;
        this.big = context.getReceiver(transform.outputs.big);
        this.all = context.getReceiver(transform.outputs.all);
    }
    process(wvalue) {
        const a = this.all.receive(wvalue);
        if (a !== operators.NonPromise)
            throw new Error("Unexpected promise: " + a);
        if (wvalue.value.length > 3) {
            const b = this.big.receive(wvalue);
            if (b !== operators.NonPromise)
                throw new Error("Unexpected promise: " + b);
        }
        return operators.NonPromise;
    }
    async startBundle() { }
    async finishBundle() { }
}
operators.registerOperator(PARTITION_URN, Partition);
////////// The actual tests. //////////
describe("worker module", (_tct_transform_1 = function () {
    var _tct_transform_6, _tct_transform_7;
    it("operator construction", (_tct_transform_6 = async function () {
        const descriptor = {
            id: "",
            transforms: {
                // Note the inverted order should still be resolved correctly.
                y: makePTransform(RECORDING_URN, { input: "pc1" }, { out: "pc2" }),
                z: makePTransform(RECORDING_URN, { input: "pc2" }, {}),
                x: makePTransform(CREATE_URN, {}, { out: "pc1" }, ["a", "b", "c"]),
            },
            pcollections: {},
            windowingStrategies: {},
            coders: {},
            environments: {},
        };
        Recording.reset();
        const processor = new worker.BundleProcessor(descriptor, null, null, [
            CREATE_URN,
        ]);
        await processor.process("bundle_id");
        assert.deepEqual(Recording.log, [
            "z.startBundle()",
            "y.startBundle()",
            "y.process(a)",
            "z.process(a)",
            "y.process(b)",
            "z.process(b)",
            "y.process(c)",
            "z.process(c)",
            "y.finishBundle()",
            "z.finishBundle()",
        ]);
    }, _tct_transform_6.__closure = () => ({ makePTransform, RECORDING_URN, CREATE_URN, Recording, worker, assert }), _tct_transform_6));
    it("forking operator construction", (_tct_transform_7 = async function () {
        const descriptor = {
            id: "",
            transforms: {
                // Note the inverted order should still be resolved correctly.
                all: makePTransform(RECORDING_URN, { input: "pcAll" }, {}),
                big: makePTransform(RECORDING_URN, { input: "pcBig" }, {}),
                part: makePTransform(PARTITION_URN, { input: "pc1" }, { all: "pcAll", big: "pcBig" }),
                create: makePTransform(CREATE_URN, {}, { out: "pc1" }, [
                    "a",
                    "b",
                    "ccccc",
                ]),
            },
            pcollections: {},
            windowingStrategies: {},
            coders: {},
            environments: {},
        };
        Recording.reset();
        const processor = new worker.BundleProcessor(descriptor, null, null, [
            CREATE_URN,
        ]);
        await processor.process("bundle_id");
        assert.deepEqual(Recording.log, [
            "all.startBundle()",
            "big.startBundle()",
            "all.process(a)",
            "all.process(b)",
            "all.process(ccccc)",
            "big.process(ccccc)",
            "big.finishBundle()",
            "all.finishBundle()",
        ]);
    }, _tct_transform_7.__closure = () => ({ makePTransform, RECORDING_URN, PARTITION_URN, CREATE_URN, Recording, worker, assert }), _tct_transform_7));
}, _tct_transform_1.__closure = () => ({ it, makePTransform, RECORDING_URN, CREATE_URN, Recording, worker, assert, PARTITION_URN }), _tct_transform_1));
//# sourceMappingURL=worker_test.js.map